{-# LANGUAGE RecordWildCards, NamedFieldPuns, ScopedTypeVariables, DeriveDataTypeable, DeriveGeneric, TemplateHaskell, FlexibleContexts #-}

{-|

Module: Digiostelium.QueryResult

Description : Data from querying agents

Copyright   : © 2017 W. Garrett Mitchener

License     : GPL-3

Maintainer  : garrett.mitchener@gmail.com

Stability   : experimental

Portability : non-portable (GHC, POSIX)

Possible results of querying an agent.  These datatypes are
declared in their own module to avoid name clashes with
"QueryRequest".

-}

module Digiostelium.OldQueryResult where

import GHC.Generics

import Data.Aeson
import Data.Aeson.TH
import Data.Binary
import Data.Data
import Data.IntMap (IntMap)
-- import qualified Data.IntMap as IM
-- import Data.Map (Map)
-- import qualified Data.Map as Map
import Data.Set (Set)
-- import qualified Data.Set as Set
import Data.Typeable
import Prelude hiding (id)

import BaselMachine.Compiler
import BaselMachine.EfficaciousAnalysis
import BaselMachine.Instruction

import Data.BitString

import Digiostelium.AesonUtil
import Digiostelium.Agent
import Digiostelium.Genome
import Digiostelium.InputOutputProblem
import Digiostelium.Problem
-- import Digiostelium.StyledNetwork



-- | Information that can be retrieved from an 'Agent'.  Normally
-- I use 'identifier' for an agent's id number so as not to clash
-- with the identity function in the prelude.  I'm using 'id'
-- here rather than 'identifier' because a lot of this stuff will
-- get exported to JSON and those extra letters could add up to a
-- lot of unnecessary data.
data AgentResult genome rating =
  JustAgent (Agent genome rating)
  | Summary {
    id :: Integer,
    rating :: rating,
    lifespan :: Lifespan,
    parentIds :: [Integer],
    genomeLength :: Integer
    }
    -- ^ quick summary: id, rating, lifespan, parents, genome length
  | Export {
    id :: Integer,
    genome :: Value
    }
    -- ^ genome information only; for moving genomes from one
    -- simulation to another, even one that solves a different
    -- problem, has different settings, etc.
  | Program {
    id :: Integer,
    rating :: rating,
    lifespan :: Lifespan,
    parentIds :: [Integer],
    genomeLength :: Integer,
    instructions :: [Instruction]
    }
    -- ^ summary of the agent plus a listing of its instructions,
    -- but no bit-level genome data; requires no significant
    -- computation
  | GenomeDetails {
    id :: Integer,
    rating :: rating,
    lifespan :: Lifespan,
    parentIds :: [Integer],
    genomeLength :: Integer,
    geneDescriptions ::  [String],
    instructionDetails :: [(Instruction, BitString, BitString, EncodingDetails)]
    -- ^ architecture-independent instruction information, @(the
    -- instruction, codon bits, nucleotide bits, encoding details)@
    }
    -- ^ summary of the agent plus a complete listing of the
    -- genome and additional information; requires no
    -- significant computation
    deriving (Show, Data, Typeable, Generic)


$(deriveJSON myJSONOpts ''AgentResult)

-- | Type alias that's really what I had in mind for 'AgentResult', but...
type AgentResultX gs problem =
  AgentResult (Genome gs) (Rating problem)
