{-# LANGUAGE DefaultSignatures, MultiParamTypeClasses #-}

{-|
Module      : Digiostelium.Snip

Description : Generating reports from agent queries

Copyright   : © 2017 W. Garrett Mitchener

License     : GPL-3

Maintainer  : garrett.mitchener@gmail.com

Stability   : experimental

Portability : non-portable (GHC, POSIX)

Type class for things that can be made into snips for nice spiffy
Pandoc documents.

-}

module Digiostelium.Snip where

import Data.Aeson (ToJSON(..))
--import qualified Data.ByteString as BS
import qualified Data.ByteString.UTF8 as U
import qualified Data.Yaml as Y
--import Text.Pandoc
import Text.Pandoc.Builder


-- | A 'Snip' object can be converted to Pandoc 'Blocks'.  The
-- @opt@ object is a way to specify options.
class Snip opt t where

  toBlocks :: opt -> t -> Blocks

  -- | The default implementation is to convert the object to
  -- JSON and write it in YAML as a code block.
  default toBlocks :: ToJSON t => opt -> t -> Blocks
  toBlocks _ x =
    codeBlock $ U.toString $ Y.encode x


-- | A 'Sniplet' object can be converted to Pandoc 'Inlines'.
-- The @opt@ object is a way to specify options.
class Sniplet opt t where

  toInlines :: opt -> t -> Inlines

  -- | The default implementation is to convert the object to
  -- YAML and write that as a line of code.
  default toInlines :: ToJSON t => opt -> t -> Inlines
  toInlines _ x =
    code $ U.toString $ Y.encode x
