{-|

Module      : Digiostelium.AesonUtil

Description : My configuration for deriving instances for Data.Aeson

Copyright   : © 2019 W. Garrett Mitchener

License     : GPL-3

Maintainer  : garrett.mitchener@gmail.com

Stability   : experimental

Portability : non-portable (GHC, POSIX)

-}

module Digiostelium.AesonUtil where

import Control.Monad
import Data.Aeson
import qualified Data.Aeson.Parser as JP
import qualified Data.Aeson.Types as JT
import qualified Data.Attoparsec.ByteString as AP
import qualified Data.ByteString.UTF8 as BSU
import Data.Char
import qualified Data.HashMap.Strict as HashMap

myJSONOpts :: Options

myJSONOpts = defaultOptions {
  sumEncoding = ObjectWithSingleField,
  fieldLabelModifier = firstToLower
  }


firstToLower :: String -> String

firstToLower [] = []
firstToLower (x:xs) = toLower x : xs



-- | Parse a 'String' into a JSON value.  It can be a leaf item
-- rather than a proper JSON document, which has to be an object
-- or an array.
stringToJSONValue :: String -> Either String JT.Value
stringToJSONValue s = AP.parseOnly JP.value (BSU.fromString s)



-- | Merge 'Value' objects, with the object on the left taking
-- precedence.  For most branches of the 'Value' datatype, this
-- is just the object on the left, but for an 'Object', the
-- fields are merged.
--
-- The choice of left-bias here is for harmony with HashMap.union
-- and Map.union.

mergeJSONTree :: JT.Value -> JT.Value -> JT.Value

mergeJSONTree (JT.Object left) (JT.Object right) =
  JT.Object (HashMap.unionWithKey (const mergeJSONTree) left right)

mergeJSONTree x _ = x


-- | Merge 'Value' objects as in 'mergeJSONTree', but either or
-- both may be 'Nothing'.  Merging 'Just object' with 'Nothing'
-- yields 'Just object'.

mergeJSONTreeM :: Maybe JT.Value -> Maybe JT.Value -> Maybe JT.Value

mergeJSONTreeM mLeft mRight =
  (liftM2 mergeJSONTree mLeft mRight) `mplus` mLeft `mplus` mRight
