{-# LANGUAGE TemplateHaskell, DeriveDataTypeable, DeriveGeneric #-}

{-|

Module: Digiostelium.ArchivePredicate

Description : Predicates for when to store an agent in the archive.

Copyright   : © 2019 W. Garrett Mitchener

License     : GPL-3

Maintainer  : garrett.mitchener@gmail.com

Stability   : experimental

Portability : non-portable (GHC, POSIX)

This is in a separate module because it's much easier to use
@derive makeBinary ...@ and avoid name clashes between @get@ and
@put@ in "Data.Binary" and "Control.Monad.Trans.State"
-}


module Digiostelium.ArchivePredicate where

import GHC.Generics

import Data.Aeson.TH
import Data.Binary
import Data.Data
import Data.Typeable

import Digiostelium.AesonUtil

-- | Which 'Agent's to keep in the Archive database

data ArchivePredicate =
  BornOnMultiple Integer
  | DiedOnMultiple Integer
  | AliveOnMultiple Integer
    deriving (Read, Show, Eq, Ord, Data, Typeable, Generic)

instance Binary ArchivePredicate
$(deriveJSON myJSONOpts ''ArchivePredicate)
