{-# LANGUAGE RecordWildCards, NamedFieldPuns, ScopedTypeVariables, 
DeriveDataTypeable, StandaloneDeriving,
FlexibleInstances, GADTs, ExistentialQuantification #-}

-- Based very much on Control.Exception

module Digiostelium.EventHandler (
  Event(..),
  SomeEvent(..),
  Receiver(..),
  dispatch
  ) where

import Data.Data
--import Data.Typeable
--import Text.Printf


-- This is basically a way of safely down-casting a data object
-- known to implement Event
class (Show e, Typeable e) => Event e where
  toSomeEvent :: e -> SomeEvent
  toSomeEvent = SomeEvent

  fromSomeEvent :: SomeEvent -> Maybe e
  fromSomeEvent (SomeEvent e) = cast e

-- This is basically a box for holding an 'Event' dictionary
data SomeEvent = forall e. Event e => SomeEvent e

deriving instance Show SomeEvent

deriving instance Typeable SomeEvent

instance Event SomeEvent

data Receiver m t = forall e. Event e => Receiver {receiveEvent :: e -> m t}

dispatch0 :: Monad m => [Receiver m t] -> SomeEvent -> m [t]
dispatch0 [] _ = return []
dispatch0 (Receiver {..} : moreReceivers) event = do
  -- The Event dictionary is pulled out of the Receiver {..}
  -- here, and this is where the fromSomeEvent in this case block
  -- comes from.
  result <- case fromSomeEvent event of
    Just decodedEvent -> do
      r <- receiveEvent decodedEvent
      return [r]
    Nothing ->
      return []
  moreResults <- dispatch0 moreReceivers event
  return (result ++ moreResults)

-- | Dispatch an 'Event' to every 'Receiver' in a list that can
-- recieve it.
dispatch :: (Monad m, Event e) => [Receiver m t] -> e -> m [t]
dispatch receivers generalEvent =
  dispatch0 receivers (toSomeEvent generalEvent)
