{-# LANGUAGE NoMonomorphismRestriction, RankNTypes, ScopedTypeVariables #-}

{-|
Module      : Digiostelium.Logging

Description : Logging utilities

Copyright   : © 2019 W. Garrett Mitchener

License     : GPL-3

Maintainer  : garrett.mitchener@gmail.com

Stability   : experimental

Portability : non-portable (GHC, POSIX)

-}

module Digiostelium.Logging where

import Control.Monad.IO.Class
import qualified Control.Exception as E
import Data.List
import GHC.Stack
import System.Log.Logger


loggerName :: String
loggerName = "Digiostelium-Dev"

type LogMessager = forall m. MonadIO m => String -> m ()

msgDebug :: LogMessager
msgDebug msg = liftIO $ debugM loggerName msg

msgInfo :: LogMessager
msgInfo msg = liftIO $ infoM loggerName msg

msgWarning :: LogMessager
msgWarning msg = liftIO $ warningM loggerName msg


msgError :: LogMessager
msgError msg = liftIO $ errorM loggerName msg

msgExc :: E.Exception e => e -> LogMessager
msgExc e msg = do
  msgError msg
  E.throw e


-- | Permform @axn@.  If an exception is raised, call 'msgError'
-- with a message indicating the @place@ and displaying the
-- exception.
excMsg :: String -- ^ @place@
          -> IO a -- ^ @axn@
          -> IO a
excMsg place axn = do
  mRes <- E.try axn
  case mRes of
    Right result ->
      return result
    Left (e :: E.SomeException) -> do
      msgError ("At " ++ place ++ " caught " ++ show e)
      cs <- currentCallStack
      msgError (intercalate "\n" cs)
      E.throwIO e
