# README -- GECCO 2021

Copyright 2021 by W. Garrett Mitchener
<mitchenerg@cofc.edu>

This is a brief document explaining how to build and use the Digiostelium artificial life simulation, as in my poster for GECCO 2021.

Conference materials referred to this as a "poster" even though the whole conference was virtual.
I'll do the same here.

## Disclaimers

This is a research project, a work in progress, and the R&D team consists of just me.
So yes, the documentation is incomplete, features are missing or not quite ready, I know, I'm working on it.

The simulation is designed to run a variety of evolutionary computations.
In this document, I describe only the essential features for running the coincidence detector problem.

This software comes with no warranty of any kind.
If it breaks you get to keep both pieces.


## How to install

These instructions are for Linux users.
Sorry, I haven't had time to work out instructions for Windows or OS/X.

My environment is Fedora 33, with the following key packages installed:
```
- gcc-10.2.1-9.fc33.x86_64
- blas-3.9.0-5.fc33.x86_64
- blas-devel-3.9.0-5.fc33.x86_64
- lapack-3.9.0-5.fc33.x86_64
- lapack-devel-3.9.0-5.fc33.x86_64
- pcre-8.44-2.fc33.x86_64
- pcre-devel-8.44-2.fc33.x86_64
- libdb-5.3.28-45.fc33.x86_64
- libdb-devel-5.3.28-45.fc33.x86_64
- gmp-6.2.0-5.fc33.x86_64
- gmp-devel-6.2.0-5.fc33.x86_64
- libgfortran-10.2.1-9.fc33.x86_64
- libquadmath-10.2.1-9.fc33.x86_64
- libgcc-10.2.1-9.fc33.x86_64
- haskell-platform-2020.2-6
```
Similar packages should be available on Debian-based systems, such as Ubuntu.

Installing the `haskell-platform` package should install the Haskell build tool `stack`.

Currently, the package builds under stackage resolver `lts-16.31`

## Build

In a terminal window, change to the directory containing `stack.yaml` and the source files.
Run the command

    stack build

This will take quite a while, as it will download and compile a *lot* of auxiliary packages from the Haskell stackage server.
You will see a lot of output in the terminal as GHC produces colorful warnings about various little things in all these packages that really ought to be updated.

It is possible to run `stack install` to install the simulation executables to a standard directory in your account.
I normally don't do this, because I have production and development versions happening in different directories and I don't want to get them confused.
If you choose to do that, assuming your `PATH` environment variable is set up properly, you can run the simulation engine as

    Engine ...


To run the simulation executable "in place" from any directory within this directory, use the command

    stack exec Engine -- ...

That is, `Engine` is the main executable.
Everything after the `--` will be passed to `Engine` rather than the `stack` build tool.

You can use

    stack exec Engine -- --help

to get help screens for the various things `Engine` can do.


## Running a sample

Change into the `CD` directory (CD, as in coincidence detector).

`Engine` reads various configuration files.
The sub-directory `GECCO-Example` includes all the ones needed for the coincidence detection task in my poster presentation.
The initial state of the sample used to produce the figure in the article is in `GECCO-Example/Samples/007`.
To run it,

    cd CD
    stack exec Engine -- reset GECCO-Example/Samples/007
    stack exec Engine -- run GECCO-Example/Samples/007

I suggest adding

    stack exec Engine -- run GECCO-Example/Samples/007 +RTS -N4

which tells the Haskell run-time system to use 4 cores for parallel processing.

It will take a long time.

If you need to stop it, hit `Control+C` in the terminal, or use `kill` to send it a signal, then wait about 30 seconds for it to exit cleanly.
You can also hit `Control+C` again and it will stop immediately, but there is a small chance the sample's files will be left in an inconsistent state.

Once it's finished, you can view the results.
To see a list of "landmark" agents, meaning the first ones to achieve a new maximum score:

    stack exec Engine -- query GECCO-Example/Samples/007

To view the agent used to generate the picture:

    stack exec Engine -- query GECCO-Example/Samples/007 --by-id 7298282

To view the efficacious instructions, append to the above `query` command:

    ... --efficacious

To view its performance on all the training tasks, append

    ... --rating-inputs

To create the diagram, append

    ... --reaction-network --graphml --path-template 'Agent-%d.graphml'

You'll have to use `yEd` <https://www.yworks.com/products/yed> to clean up the GraphML diagram and straighten out the input-output arrows.

To make a new sample, say `GECCO-Example/Samples/foo`, copy the `DB_CONFIG` file from `007` to that directory, then run

    stack exec Engine -- init GECCO-Example/Samples/foo
    stack exec Engine -- run GECCO-Example/Samples/foo



## More usage notes

- Every so often, `Engine` writes a quick summary to `Status.yaml` in the sample directory.
  You can view this to monitor its progress.

- For reasons related to maintaining the integrity of the database files, `Engine` creates a couple of lock files in the sample directory while it's working.
  Another `Engine` process will not do anything in a sample directory that is locked.
  If `Engine` crashes or is violently killed, it may leave these files behind.
  You may remove `GlobalLock.txt` and `Process*.txt` to unlock a sample if that happens, however, the database files may be corrupted.
  If `Engine` displays `DB` error messages about pages not found and so on, this is likely the case.
  Berkeley DB (libdb) does come with recovery tools, but I haven't had much luck with them.

- So, to aid in recovering from crashes, every 15 minutes, `Engine run` pauses and makes a snapshot.
  It copies the important database files to a subdirectory `Snapshot/0`, and moves old snapshots up a number.
  Only the most recent three are kept.
  If a sample becomes corrupted, you can try copying the files from `Snapshot/0` into the sample directory, and pick up from where it was 15 minutes ago.
  You could also try `Snapshot/1` and `Snapshot/2`.

- Incidentally, you can run queries on snapshots, as in

      stack exec Engine -- query GECCO-Example/Samples/007/Snapshot/0

  This is useful for seeing more details about a sample's progress, since you cannot query the main sample directory while `Engine run` is working on it.


There are a few useful scripts in the `CD` subdirectory.

Say you're running a batch of samples, in directories `MyBatch/Samples/000`, `MyBatch/Samples/001`, etc.

To see a summary based on the `Status.yaml`, run

    ./ProgressCheck MyBatch

To check the database files, and recover from crashes as best you can, run

    ./IntegrityCheck MyBatch

To interactively look at and clear out a bunch of directory locks,

    ./ClearAllLocks

To clear them all out automatically,

    ./ClearAllLocks go


## Summary of source code

`BaselMachine-0.8`: This package is the virtual machine that runs each agent's instructions.
It includes an arena framework that allows agents to interact, but this is not used in the coincidence detection problem.

`Digiostelium-0.8`: This package is where most of the simulation code resides.
The file `CoincidenceDetectorProblem.hs` is a good place to start if you want to see how all the tasklets are built and exactly how everything is scored.

`Sim-0.8`: The `Engine` main module, plus a few other executables used on other projects.

`bitutil-0.8`: Fast bit string library

`hprandom-0.8`: Haskell wrapper for a high performance Mersenne Twister random number generator

`FileMutex-0.8`: How to lock directories

`wgmutils-0.8`: Various odds and ends that don't go anywhere else

`BerkeleyDB-0.8.7`: Haskell wrapper around Berkeley DB library
