{-# LANGUAGE GeneralizedNewtypeDeriving, StandaloneDeriving #-}

{- |

Module: WGM.OrphanInstances

Description : Orphan instances that are necessary to get other things working

Copyright   : © 2017 W. Garrett Mitchener

License     : GPL-3

Maintainer  : garrett.mitchener@gmail.com

Stability   : experimental

Portability : non-portable (GHC, POSIX)


-}

module WGM.OrphanInstances where

import GHC.Generics

import Data.Aeson
import Data.Binary
import Data.Semigroup
import qualified Data.Vector.Unboxed as UV


deriving instance ToJSON t => ToJSON (Sum t)
deriving instance FromJSON t => FromJSON (Sum t)
deriving instance ToJSONKey t => ToJSONKey (Sum t)
deriving instance FromJSONKey t => FromJSONKey (Sum t)

deriving instance ToJSON t => ToJSON (Product t)
deriving instance FromJSON t => FromJSON (Product t)
deriving instance ToJSONKey t => ToJSONKey (Product t)
deriving instance FromJSONKey t => FromJSONKey (Product t)

deriving instance ToJSONKey t => ToJSONKey (Last t)
deriving instance FromJSONKey t => FromJSONKey (First t)
deriving instance ToJSONKey t => ToJSONKey (First t)
deriving instance FromJSONKey t => FromJSONKey (Last t)



instance (UV.Unbox t, Binary t) => Binary (UV.Vector t) where

  put v = do
    put (UV.length v)
    UV.mapM_ put v

  get = do
    n <- get
    UV.replicateM n get
