{-# LANGUAGE FlexibleInstances, FlexibleContexts, UndecidableInstances #-}

{-|

Module: WGM.ShortShow

Description: A class for converting objects into abbreviated
string form for display.  Unlike the standard 'show', there is no
expectation that the result of 'shortShow' can be read in any
way.

Copyright: © W. Garrett Mitchener 2017

License: GPL-3

Maintainer: garrett.mitchener@gmail.com

Stability: experimental

Portability: Portable

-}

module WGM.ShortShow where

import Text.Printf
import Data.List


class Show t => ShortShow t where

  shortShow :: t -> String

  shortShow = show


instance ShortShow Int

instance ShortShow Double where
  shortShow x = printf "%.2f"

instance ShortShow String where
  shortShow x =
    if length x <= 10
    then show x
    else
      printf "\"%9s…\"" x

instance ShortShow t => ShortShow [t] where
  shortShow xs =
      "["
      ++ intercalate "," (map shortShow (take 10 xs))
      ++ (if length xs <= 10
          then "]"
          else ",…]")

instance (ShortShow t1, ShortShow t2) => ShortShow (t1, t2) where
  shortShow (x1, x2) =
    concat ["(", shortShow x1, ",", shortShow x2, ")"]
